﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/10/30 初版
//
//-------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
//FVXデータ構造の使用宣言
using FVDATASLib;


namespace Ryudo
{
	public partial class frmMain : Form
	{
		public frmMain()
		{
			InitializeComponent();
		}

		//処理範囲
		FvRectangle m_ExeRegion = new FvRectangle();
		// ビデオデバイスがオープンされているか
		private bool m_blnVideoOpened = false;
		// ビデオ設定ファイル名（フルパス）
		private string m_strVideoConfFilePath = "";
		// 初期ディレクトリ
		private string m_strDirectory;

		private void FormMain_Load( object sender, EventArgs e )
		{
			int IntErr;
			frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
			frmLicense fLicense = new frmLicense();

			//色設定
			cmbColor.Items.Insert(0, "BLACK");
			cmbColor.Items.Insert(1, "WHITE");
			cmbColor.SelectedIndex = 0;

			//処理範囲の初期値設定(画像の全面に設定)
			m_ExeRegion.xs = -1;
			m_ExeRegion.xe = -1;
			m_ExeRegion.ys = -1;
			m_ExeRegion.ye = -1;

			//作業ディレクトリ退避
			m_strDirectory = Directory.GetCurrentDirectory();

			//使用許諾の確認
			if ( fLicense.ShowDialog() == DialogResult.OK )
			{
				//ビデオ設定ファイルの選択
				if ( fVideoConf.ShowDialog() == DialogResult.OK )
					m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

				if ( fVideoConf.chkNotUseCamera.Checked == false )
				{
					//ビデオ設定ファイルをコンポーネントに設定
					axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

					//デバイスオープン
					try
					{
						axFvxVideo1.OpenDevice();
					}
					catch ( Exception )
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
					//このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
					//本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
					//お使いのキャプチャボードに合わせて変更してください。
					System.Threading.Thread.Sleep(350);

					//GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
					//あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
					//（取り込みサイズとImageのサイズが違う場合エラーになります）
					axFvxVideo1.ChangeImageSize(axFvxImageGray.GetOcx());
					try
					{
						// 同期取込
						axFvxVideo1.GrabSync(axFvxImageGray.GetOcx());
					}
					catch ( Exception )
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//ビデオデバイスの状態を保存
					m_blnVideoOpened = true;
					btnGrab.Enabled = true;
				}
				//画面の生成
				axFvxDisplayEx1.CreateDisplay();
				//画像の表示
				axFvxDisplayEx1.DrawImage(axFvxImageGray.GetOcx());
			}
			else
			{
				//終了
				Application.Exit();
				return;
			}
		}

		//終了時
		private void frmMain_FormClosed( object sender, FormClosedEventArgs e )
		{
			if ( m_blnVideoOpened )
				axFvxVideo1.CloseDevice();
		}

		//カメラ取込
		private void btnGrab_Click( object sender, EventArgs e )
		{
			//画像ファイル読み込みによって、画像サイズが初期化時から
			//変更されている可能性があるため、ChangeImageSizeを行います。
			//画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
			axFvxVideo1.ChangeImageSize(axFvxImageGray.GetOcx());

			try
			{
				//同期取込
				axFvxVideo1.GrabSync(axFvxImageGray.GetOcx());
			}
			catch ( Exception )
			{
				int IntErr;
				axFvxVideo1.GetErrReport(out IntErr);
				MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
			}
			//画像の表示
			axFvxDisplayEx1.DrawImage(axFvxImageGray.GetOcx());

		}

		private void btnLoadImage_Click( object sender, EventArgs e )
		{
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			openFileDialog1.DefaultExt = "*.bmp";
			openFileDialog1.ShowDialog();

			if ( openFileDialog1.FileName.Length > 0 )
			{

				//FvxFileAccessを用いてファイルからFvxImageに画像を読込
				axFvxFileAccess1.FilePath = openFileDialog1.FileName;

				try
				{
					axFvxFileAccess1.LoadBitmap(axFvxImageGray.GetOcx());
				}
				catch ( Exception )
				{
					int intErr;
					axFvxFileAccess1.GetErrReport(out intErr);
					MessageBox.Show("読込に失敗しました。(ErrCode=" + intErr.ToString() + ")", "読込失敗");
				}
				//画像タイプチェック
				if ( axFvxImageGray.BitPerPixel != 8 || axFvxImageGray.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray )
				{
					MessageBox.Show("8bitグレイの画像ではありません", "error");
                    axFvxImageGray.BitPerPixel = 8;
					axFvxImageGray.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
					axFvxImageGray.ImageWidth = 512;
					axFvxImageGray.ImageHeight = 480;
				}

				//画像表示
				axFvxDisplayEx1.DrawImage(axFvxImageGray.GetOcx());
			}
		}

		//画像保存
		private void btnSaveImage_Click( object sender, EventArgs e )
		{
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			saveFileDialog1.DefaultExt = "*.bmp";
			saveFileDialog1.ShowDialog();

			if ( saveFileDialog1.FileName.Length > 0 )
			{
				//FileAccessコンポーネントを使って
				//Imageをファイルに保存します。
				axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
				try
				{
					axFvxFileAccess1.SaveBitmap(axFvxImageGray.GetOcx());
				}
				catch ( Exception )
				{
					int intErr;
					axFvxFileAccess1.GetErrReport(out intErr);
					MessageBox.Show("保存に失敗しました。　ErrCode=" + intErr.ToString());
					return;
				}
			}
		}

		//処理範囲設定
		private void btnExeRegion_Click( object sender, EventArgs e )
		{
			int IntOK = 0;
			double dblTemp = 0.0;

			axFvxROI1.SetRectangle(m_ExeRegion);

			// FvxROIを使って処理範囲選択ダイアログを出します｡
			axFvxROI1.UsePoint = false;
			axFvxROI1.MakeROIDialog(axFvxImageBin.GetOcx(), out IntOK);

			if ( ( IntOK == 1 ) && ( axFvxROI1.CheckRectangle == 1 ) )
			{
				axFvxROI1.GetRectangle(m_ExeRegion);
				// 始点と終点が逆になっていないかチェックします｡
				// FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				// 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if ( m_ExeRegion.xe < m_ExeRegion.xs )
				{
					dblTemp = m_ExeRegion.xs;
					m_ExeRegion.xs = m_ExeRegion.xe;
					m_ExeRegion.xe = dblTemp;
				}
				if ( m_ExeRegion.ye < m_ExeRegion.ys )
				{
					dblTemp = m_ExeRegion.ys;
					m_ExeRegion.ys = m_ExeRegion.ye;
					m_ExeRegion.ye = dblTemp;
				}
			}
		}

		//表示のクリア
		private void btnClear_Click( object sender, EventArgs e )
		{
			txtResult.Clear();
			axFvxDisplayEx1.ClearOverlay();
			axFvxImageGray.GetOcx();
		}

		//2値化試行
		private void btnApplyBinLevel_Click( object sender, EventArgs e )
		{
			int intBinLevel;
			if ( int.TryParse(txtBinLevel.Text, out intBinLevel) == false || 255 < intBinLevel )
			{
				txtBinLevel.Text = "128";
				intBinLevel = 128;
			}
			if ( axFvxImageGray.BitPerPixel == 8 )
			{
				//ChangeImageを利用してグレイImageから2値Imageを作る
				axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), intBinLevel);
			}
			else
			{
				MessageBox.Show("8bit濃淡画像ではありません");
				axFvxImageGray.CopyTo(axFvxImageBin.GetOcx());
			}
			axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());
		}

		//2値計測実行
		private void btnExecute_Click( object sender, EventArgs e )
		{
			double dblTime;		//処理時間
			int intErr;			//エラー番号
			int intBinLevel;	//2値化閾値
			int intMinSize;		//最小面積
			int intMaxSize;		//最大面積
			int intDivideNum;	//面積範囲分割数
			int intCount;		//見つかったblobの個数
			FvBlobResultGroup pBlobresultGroup = new FvBlobResultGroup();	//2値計測結果

			//画面消去
			axFvxDisplayEx1.ClearOverlay(-1);

			//分割数チェック
			if ( int.TryParse(txtDivideNum.Text, out intDivideNum) == false || intDivideNum < 2 )
			{
				MessageBox.Show("分割数は2以上の数値を指定して下さい", "パラメータエラー");
				return;
			}
			//最大・最小面積値チェック
			if ( int.TryParse(txtMinSize.Text,out intMinSize) == false || intMinSize < 1 )
			{
				MessageBox.Show("最小面積値は1以上の数値を設定して下さい", "パラメータエラー");
				return;
			}
			if ( int.TryParse(txtMaxSize.Text,out intMaxSize)  == false || intMinSize >= intMaxSize)
			{
				MessageBox.Show("最小面積値＜最大面積値となるよう設定して下さい", "パラメータエラー");
				return;
			}

			//始点と終点も必要なので、分割数+1個変数が必要
			int[] intSizeStep = new int[intDivideNum+1];	//面積分割の閾となる値
			int[] intBlobNum = new int[intDivideNum+1];		//分割された面積範囲ごとのblobの数

			//面積分割地点の計算
			for ( int i = 0; i < intDivideNum; i++ )
			{
				intSizeStep[i] = intMinSize + ( intMaxSize - intMinSize ) * i / intDivideNum;
			}

			//最大・最小面積設定
			axFvxBlob1.MinSize = intMinSize;
			axFvxBlob1.MaxSize = intMaxSize;

			//対象職設定
			if ( cmbColor.Text == "BLACK" )
				axFvxBlob1.Color = FVXCOMBLOBLib.COLOR_TYPE.BLACK_COLOR;
			else
				axFvxBlob1.Color = FVXCOMBLOBLib.COLOR_TYPE.WHITE_COLOR;

			//ラベリング（検出順序）設定
			RadioButton radio = new RadioButton();
			foreach ( Control control in groupBoxArrange.Controls )
			{
				if ( control.GetType() == radio.GetType() )
				{
					radio = ( RadioButton )control;
					if ( radio.Checked == true )
					{
						axFvxBlob1.Arrange = ( FVXCOMBLOBLib.ARRANGE_TYPE )Convert.ToInt32(radio.Tag);
					}
				}
			}

			//処理範囲接触許可
			if ( chkWindowTouch.Checked == true )
				axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.ON ;
			else
				axFvxBlob1.WindowTouch = FVXCOMBLOBLib.WINDOW_TOUCH_TYPE.OFF;

			//閾値チェック
			if (int.TryParse(txtBinLevel.Text,out intBinLevel) == false || intBinLevel < 0)
				intBinLevel = 0;	
			if ( intBinLevel > 255 )
				intBinLevel = 255;
			
			//2値化
			if ( axFvxImageGray.BitPerPixel == 8 )
				axFvxChangeImage1.FixedLevel(axFvxImageGray.GetOcx(), axFvxImageBin.GetOcx(), intBinLevel);
			else if ( axFvxImageGray.BitPerPixel == 1 )
				axFvxImageGray.CopyTo(axFvxImageBin.GetOcx());
			else
			{
				MessageBox.Show("BPP =1/8以外の画像が入力されました", "入力画像エラー");
				return;
			}
			axFvxDisplayEx1.DrawImage(axFvxImageBin.GetOcx());

			//処理範囲設定
			axFvxBlob1.SetInspectArea(m_ExeRegion);
			//処理範囲を赤で描画
			axFvxDisplayEx1.PenColor = Color.Red;
			axFvxDisplayEx1.DrawData(0, m_ExeRegion);

			try
			{
				//時間測定開始
				axFvxPerformanceCounter1.StartCount();

				//2値計測実行
				axFvxBlob1.Execute(axFvxImageBin.GetOcx(), out intCount);

				//時間計測終了
				axFvxPerformanceCounter1.StopCount(out dblTime);

				//結果の取得
				//xxResultGroupにて結果を受け取る場合には、第一引数は無視されます。
				axFvxBlob1.GetResult(0, pBlobresultGroup);

				//重心点と外接長方形を描画します
				axFvxDisplayEx1.DrawData(0, pBlobresultGroup);

				int i, j;
				for ( i = 0; i < pBlobresultGroup.Count; i++ )
				{	//Blobがどの面積班員に属しているか調べ、その数を記録します。
					for ( j = 0; j < intDivideNum; j++ )
					{
						if ( ( intSizeStep[j] <= pBlobresultGroup.Item2(i).area ) && ( pBlobresultGroup.Item2(i).area < intSizeStep[j + 1] ) )
							intBlobNum[j] = intBlobNum[j] + 1;
					}
				}
				//結果をテキストボックスに表示します。
				txtResult.Text = "処理時間 : " + dblTime.ToString("0.00ms") + "\r\n" +
								"検出個数 : " + intCount.ToString() + "\r\n" + "\r\n" +
								"面積範囲　　　　　　個数" + "\r\n";
				for ( j = 0; j < intDivideNum; j++ )
				{
					txtResult.Text = txtResult.Text + intSizeStep[j].ToString("0").PadLeft(6) + "～" + intSizeStep[j + 1].ToString("0").PadLeft(6) + " : " + intBlobNum[j].ToString("0").PadLeft(4) + "\r\n";
				}
			}
			catch ( COMException )
			{
				axFvxBlob1.GetErrReport(out intErr);
				MessageBox.Show("2値計測エラー errcode= " + intErr.ToString());
				return;
			}
		}

	}
}